/**
 * Copyright (c) 2000-2013 Liferay, Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 */

package com.liferay.nativity.modules.fileicon.win;

import com.liferay.nativity.Constants;
import com.liferay.nativity.control.MessageListener;
import com.liferay.nativity.control.NativityControl;
import com.liferay.nativity.control.NativityMessage;
import com.liferay.nativity.modules.fileicon.FileIconControlBase;
import com.liferay.nativity.modules.fileicon.FileIconControlCallback;
import com.liferay.nativity.util.win.RegistryUtil;

import java.util.List;
import java.util.Map;

/**
* @author Dennis Ju
*/
public class WindowsFileIconControlImpl extends FileIconControlBase {

	public WindowsFileIconControlImpl(
		NativityControl nativityControl,
		FileIconControlCallback fileIconControlCallback) {

		super(nativityControl, fileIconControlCallback);

		MessageListener messageListener = new MessageListener(
			Constants.GET_FILE_ICON_ID) {

			@Override
			public NativityMessage onMessage(NativityMessage message) {
				String filePath = null;

				if (message.getValue() instanceof List<?>) {
					List<?> args = (List<?>)message.getValue();

					if (args.size() > 0) {
						filePath = args.get(0).toString();
					}
				}
				else {
					filePath = message.getValue().toString();
				}

				int icon = getIconForFile(filePath);

				return new NativityMessage(Constants.GET_FILE_ICON_ID, icon);
			}
		};

		nativityControl.registerMessageListener(messageListener);
	}

	@Override
	public void disableFileIcons() {
		RegistryUtil.writeRegistry(
			Constants.NATIVITY_REGISTRY_KEY,
			Constants.ENABLE_OVERLAY_REGISTRY_NAME, 0);
	}

	@Override
	public void enableFileIcons() {
		RegistryUtil.writeRegistry(
			Constants.NATIVITY_REGISTRY_KEY,
			Constants.ENABLE_OVERLAY_REGISTRY_NAME, 1);
	}

	@Override
	public int registerIcon(String path) {
		return 0;
	}

	@Override
	public void removeAllFileIcons() {
	}

	@Override
	public void removeFileIcon(String path) {
	}

	@Override
	public void removeFileIcons(String[] paths) {
	}

	@Override
	public void setFileIcon(String path, int iconId) {
	}

	@Override
	public void setFileIcons(Map<String, Integer> fileIconsMap) {
	}

	@Override
	public void unregisterIcon(int id) {
	}

}